// GPS Track Class ********************************************************
class GPSTrack {
  GPSPoint[] points;
  int bornFrame;
  color trackColor;

  GPSTrack(int numpoints) {
    points = new GPSPoint[numpoints];
    trackColor = color(255, 255, 255, 255);
  }

  GPSTrack(GPSTrack track) {
    points = new GPSPoint[track.points.length];
    arrayCopy(track.points, points);
    bornFrame = -1;
    trackColor = track.trackColor;
  }

  // 2D drawing functions
  void drawTrack2D(boolean breaksegs) {   // Draw the whole track, boolean flag to breaksegs (false=no breaks, true=breaks)
    beginShape();
    for (int pointindex=0; pointindex<points.length; pointindex++) {
      vertex(points[pointindex].longitude, points[pointindex].latitude);
      if (breaksegs == true) {
        if (pointindex != (points.length-1)) {
          if (points[pointindex].segment != points[pointindex+1].segment) {
            endShape();
            beginShape();
          }
        }
      }
    }
    endShape();
  }

  void drawTrack2D(int startpoint, int endpoint, boolean breaksegs) {   // Draw the whole track, boolean flag to breaksegs (false=no breaks, true=breaks)
    beginShape();
    for (int pointindex=startpoint; pointindex<endpoint; pointindex++) {
      vertex(points[pointindex].longitude, points[pointindex].latitude);
      if (breaksegs == true) {
        if (pointindex != (points.length-1)) {
          if (points[pointindex].segment != points[pointindex+1].segment) {
            endShape();
            beginShape();
          }
        }
      }
    }
    endShape();
  }

  void drawTimeRange(float startTime, float endTime, boolean breaksegs) { //draw all of the points between the start time and end time (measured in seconds)
    beginShape();
    for (int i=0; i<points.length; i++) {
      float pointTime = float(points[i].time);
      if (startTime < pointTime) {
        if (pointTime < endTime) {
          vertex(points[i].longitude, points[i].latitude);
          if (breaksegs == true) {
            if (i != (points.length-1)) {
              if (points[i].segment != points[i+1].segment) {
                endShape();
                beginShape();
              }
            }
          }
        } else {
          break;
        }
      }
    }
    endShape();
  }

  void drawShape(PShape shape, float startTime, float endTime, color endColor) {
    for (int i=0; i<points.length; i++) {
      float pointTime = float(points[i].time);

      if (pointTime < startTime && pointTime > endTime) {
        noFill();
        float timeSpan = startTime - pointTime;
        float timePos = map(pointTime, startTime, endTime, 0.0, 1.0);
        pushMatrix();
        translate(points[i].longitude, points[i].latitude);
        scale(timePos);
        shape.setStroke(lerpColor(trackColor, endColor, timePos));
        shape(shape); 
        popMatrix();
        //stroke(lerpColor(trackColor, endColor, timePos));
        //ellipse(points[i].longitude, points[i].latitude, timeSpan, timeSpan);
      }
    }
  }

  void drawScatter(float startTime, float endTime) {
    for (int i=0; i<points.length; i++) {
      float pointTime = float(points[i].time);

      if (pointTime < startTime && pointTime > endTime) {
        noFill();
        float timeSpan = (startTime - pointTime);
        if (timeSpan < maxRadius) {
          strokeWeight(10);
          float angle = random(TWO_PI);
          if (timeSpan < timeInterval) { //if these points are very fresh, make them a little brighter.
            stroke(color(red(trackColor), green(trackColor), blue(trackColor), 2));
            point(points[i].longitude+cos(angle)*timeSpan, points[i].latitude+sin(angle)*timeSpan);
          } else {
            float probability = timeSpan / maxRadius;
            float flip = random(.5);
            if ( flip > probability) { //only render a point if random returns a number greater than probabilty
              stroke(trackColor);
              point(points[i].longitude+cos(angle)*timeSpan, points[i].latitude+sin(angle)*timeSpan);
            }
          }
        }
      }
    }
  }

  String printPoint(int pointindex) {    // Display the values of a specific point
    String pointvalues = "Point # " + pointindex + ": " + points[pointindex].longitude + " " + points[pointindex].latitude + " " + points[pointindex].altitude + " " + points[pointindex].time + points[pointindex].segment;
    return pointvalues;
  }
}

