// Transmission 2
// Joshua Albers
// contact@joshuaalbers.com
// 2015

import org.gicentre.utils.spatial.*;
import org.gicentre.utils.geom.*;
import processing.opengl.*;
import java.text.SimpleDateFormat;
import java.util.SimpleTimeZone;
import java.util.Calendar;
import java.text.ParseException;
import java.util.List;
import java.util.Date;

float lat = 41.9;
float lon = -87.6485;

//bounding box to limit data retrieved from the database
float northLat = 42.023131;
float southLat = 41.644335;
float eastLon = -87.524044;
float westLon = -87.940267;

float camscale = 0.07;
float camx;
float camy;
float mouseStartX = 0.0;
float mouseStartY = 0.0;

float camdistbase = 1.5;
float camdist = camdistbase;
float resolutionMultiplier = 2.0;

int beadsize = 3;
int weightMultiplier = 1;
boolean guideToggle = false;
boolean wholeToggle = false; 
boolean screenshot = false;
boolean loadFinished = false;
boolean saveFrames = true;

int frameNumber = 0;

//String startDate = "2014-06-01";
String startDate = "2011-06-18";
String endDate = ""; //set to "" to use today as endDate

float currentTime = 0;
float endTime = 86400;
float timeInterval = 4; // .033 is 1/30sec, .8 is 24 hours in 1 hour, 4 is 24 hours in 12 minutes

String user = "joshua";
String pass = "";
String databasePath = "jalbers_gps_20150503031001_db";

Ellipsoid globe = new Ellipsoid(Ellipsoid.WGS_84);
UTM chiUTM = new UTM(globe, lat, lon);
WebMercator wm = new WebMercator();

ArrayList tracks = new ArrayList<GPSTrack>();


PShape circle;

float maxRadius;

//String databasePath = "/Users/joshua/Dropbox/MyTracks/jalbers_gps_20120618213743.db";
//String databasePath = "jalbers_gps_20111011151257.db";
//String csvlistfile = "chi_csv_tracks.txt";

void setup() {
  frameRate(MAX_FLOAT);
  size(1280, 720, P2D);
  //noSmooth();
  smooth();
  shapeMode(CENTER);
  maxRadius = dist(0, 0, width, height) * 5;
  println(maxRadius);
  noFill();
  stroke(255);
  circle = createShape(ELLIPSE, 0, 0, maxRadius*2, maxRadius*2);
  circle.setStrokeWeight(100);

  if (!saveFrames) {
    mouseWheelListener();
  }

  //bigBuffer = createGraphics(width * resolutionMultiplier, height * multiplier);

  //String[] csvtracklist = loadStrings(csvlistfile);
  //GPSTrack[] tracks = getChiDailySQL(databasePath);
  String loadDataThread = "getDailyGPS";
  thread(loadDataThread);


  //convert camlat and camlon values to UTM
  //  PVector cam = chiUTM.latLongToUTM(new PVector(lon, lat));
  //  camx = cam.x;
  //  camy = cam.y;
  PVector cam = wm.transformCoords(new PVector(lon, lat));
  camx = cam.x;
  camy = cam.y;
  println(camx + " "+ camy + " " + camscale);
  background(0);
}

void draw() {
  if (loadFinished) {

    colorMode(RGB);
    strokeJoin(ROUND);
    strokeCap(ROUND);
    //background(32);
    noFill();

    float aspectRatio = 1;
    pushMatrix();
    if (screenshot) setCameraView2D(camx, camy, aspectRatio, camscale, resolutionMultiplier );
    else setCameraView2D(camx, camy, aspectRatio, camscale, 1.0);

    if (tracks.size() > 0) {
      //Draw all of the lines***********************************************************************************
      for (int a=0; a<tracks.size (); a++) {
        //initialize the startpoint and endpoint variables (we will change one or more of these for each track)
        int startpoint = 0;
        int endpoint = 0;
        GPSTrack currentTrack = (GPSTrack)(tracks.get(a));

        int trackSize = currentTrack.points.length;
        int frameOffset = frameCount - currentTrack.bornFrame;

        if (wholeToggle) {
          stroke(255, 32);
          strokeWeight(1 * weightMultiplier);
          currentTrack.drawTrack2D(true);
        } else {
          strokeWeight(1 * weightMultiplier);
          noFill();
          //stroke(255, 32);
          //currentTrack.drawTimeRange(currentTime - 600, currentTime, true);
          //stroke(currentTrack.trackColor);
          //          color currentColor = currentTrack.trackColor;
          //          currentTrack.drawShape(circle, currentTime, currentTime - 46300, color(red(currentColor), green(currentColor), blue(currentColor), 0));
          //          
          currentTrack.drawScatter(currentTime, 0);
        }
        if (guideToggle) {
          stroke(255, 0, 0, 128);
          strokeWeight(10); 
          line(camx, camy, currentTrack.points[0].longitude, currentTrack.points[0].latitude);
        }
      }
      frameNumber++;
    }
    popMatrix();
    if (saveFrames) saveFrame("transmission_2_" + nf(frameNumber, 6) + ".png");
    int hours = floor(currentTime / 3600);
    int minutes = floor((currentTime % 3600) / 60);
    int seconds = floor(currentTime % 60 );
    int milliseconds = floor( (currentTime * 1000) % 1000);
    String currentTimeString = nf(hours, 2) + ":" + nf(minutes, 2) + ":" + nf(seconds, 2) + "." + nf(milliseconds, 3);
    noStroke();
    //fill(0);
    //rect(0, 0, textWidth(currentTimeString)+20, 20);
    //fill(255);
    //text(currentTimeString, 10, 10);
    println(currentTimeString);
    currentTime += timeInterval;
    if (currentTime >= endTime)
    {
      currentTime = 0;
      background(0);
      exit();
    }
  }
}

void keyPressed() {
  if (key == 'i' || key == 'I') {
    println(camx + " " + camy + " " + camscale);
  } else if (key == 'g' || key == 'G') {
    guideToggle = !guideToggle;
  } else if (key == 'w' || key == 'W') {
    wholeToggle = !wholeToggle;
  } else if (key == 'p' || key == 'P') {
    saveScreen();
  } else if (key == '1' || key == '2' || key == '3' || key == '4' || key == '5' || key == '6' || key == '7' || key == '8' || key == '9') {
    weightMultiplier = Character.getNumericValue(key);
    println(weightMultiplier);
  } else if (key == CODED) {
    if (keyCode == UP) {
      camscale *= 1.01;
    } else if (keyCode == DOWN) {
      camscale *= 0.99;
    }
  }
}

void mousePressed() {
  mouseStartX = mouseX;
  mouseStartY = mouseY;
}

void mouseDragged() {
  if (!saveFrames) {
    camx += (pmouseX - mouseX) / camscale;
    camy += (mouseY - pmouseY) / camscale;
    mouseStartX = mouseX;
    mouseStartY = mouseY;
  }
}

void mouseReleased() {
  PVector screenTarget = wm.webMercatorToLatLong(new PVector(camx, camy, 0.0));
  println(screenTarget, camscale);
}

